/* nodes.c -- this is the actual framework for managing a given node
   of the bbs. */

#include <stdio.h>
#include <unistd.h>
#include <pwd.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <errno.h>
#include <curses.h>
#include <menu.h>
#include <stdlib.h>
#include <string.h>

/* define IN_BBS so we don't collide constants. */
#define IN_BBS

#include <common.h>
#include <client.h>
#include <menu_codes.h>

#include "msgs.h"
#include "files.h"

#include "internode.h"
#include "disp.h"
#include "nodes.h"

#define DEBUG

/* initialize globals */

extern BBS_Config *config;
extern BBS_Node *this_node;
extern int my_node;
extern time_t last_checked;

/* functions that only we care about */
extern void spawn_door(BBS_Menu *);

/* prototype the local functions */

void who(void);
void new_user(void);
void mod_user(BBS_User *);
void talk_to_node(void);
void server_int(int sig);
void begin_chat_loop(int inpipe, int outpipe);
void db_shutdown(int sig);

void bbs_main(void) {

     char opt;
     char test[10];
     char logbuf[80];

/* define our node, which is now assigned by connection controls
   rather than at login time. while, for now, all the node stuff is in
   a memory space that the DB server can also access, we shouldn't
   assume it will always be like this. */

/* set up our signal handlers */

/*     signal(SIGSEGV, *clean_death);
     signal(SIGBUS, *clean_death);
     signal(SIGTERM, *clean_death);     
     signal(SIGINT, *clean_death); 
     signal(SIGQUIT, *clean_death);
     signal(SIGHUP, *clean_death);
*/
     /* Inc comms */
     signal(SIGUSR2, server_int);
     /* server death */
     signal(SIGUSR1, db_shutdown);

     /* we should have been started as root so that dosemu/doors may
	be happy, but we don't want to run with those perms all the
	time. */

     setgid(config->setgid);
     seteuid(config->setuid);

     /* we expect that this_node has beem allocated. */

     /* ncurses should have already been brought up for us. */

     move(LINES-1,0);

     /* login our user and prep the rest of the node data */
     
     if((this_node->current_user = user_login()) == NULL)
	  return;

     prep_colors(this_node->current_user);
     init_pairs(this_node->current_user);

     /* update the various node data with stats of our current process. */
     this_node->pid = getpid();
     this_node->area[0] = 0;
     this_node->avail = 2;
     this_node->node = my_node;

     printw("Connected to %s (node %d)\n", VERSION, my_node);

     sprintf(logbuf,"%s connected to node %d.\n",
	     this_node->current_user->handle, my_node);

     log(logbuf);

     update_node(this_node);     

     this_node->current_user->last_login = time(NULL);
     last_checked = time(NULL);

     update_node(this_node);

     display_file(config->motd);

     /* bring them up to the main menu, and keep up the menu loop */
     /* anyone ever notice that BTE's 'Desperately Wanting' has some
	HORRID lyrics?  e.g. "When they pumped out your guts" ?  guts
	is really just NOT a melodic word.  I'm sorry. */

     move(LINES, 0);

     strcpy(this_node->current_menu, "main");

     /* set them outside of a message or file area */
     this_node->area[0] = 0;
     this_node->filearea[0] = 0;
 
     while(menu() == 0) {
	  
	  this_node->current_user->remaining = this_node->current_user->remaining - (time(NULL) - last_checked);
	  last_checked = time(NULL);

     }

     prompt("Leave feedback for %s [y/N]? ",config->sysop);
     
     opt = getch();
     
     if(opt == 'y') {
	  strcpy(this_node->area, "Private");
	  compose_message(config->sysop, NULL);
     }

     /* everything's done, call that catch-all (hah!) clean_death */

     clean_death(0);     

     return;
     
}

/* log a user in...   it's pretty obvious..   
   NOTE: clean up the new user stuff! */
/* it really may not work at all right now. */

BBS_User *user_login(void)
{
     
     BBS_User *ret;
     char typed[20];
     char typed2[20];

     char issue[80];
     int tries=0;
     
     
     ret = NULL;
     my_node = 0;

     /* welcome to... */
     
     sprintf(issue,"%s/%s",config->base_dir,config->issue);

     if(display_file(issue) != 0)
	  bbserr("display_file bombed on %s\n",issue);
     
     /* figure out who we've got here.  give 'em 3 shots at it. */

     while ((my_node < 1) && (tries < 3)) {
	  tries++;
	  addstr("login/handle: ");
	  /* are they saying anything? */
	  if(scanw("%s",typed) < 1)
	       continue;

	  /* for now, new users don't get a node.  fuck 'em.  */
	  if(!strcasecmp(typed,"new")) {
	       new_user();
	       return NULL;
	  }

	  addstr("password: ");
	  noecho();
	  scanw("%s", typed2);
	  echo();
	  addch('\n');
	  
	  /* call out to the db server to verify the password and get
	     a  free node for ourselves. */

	  my_node = login_node(typed, typed2);

	  if(my_node == 0) {
	       bbserr("Sorry, but all %d nodes are in use. Please try later.\n", config->nodes);
	       return NULL;
	  }

	  if(my_node == -1) 
	       bbserr("Login incorrect\n");
     }
     
     if(tries >= 3) {
	  bbserr("Sorry, you have excedded the maximum number of");
	  bbserr(" login attempts.\nGoodbye.\n");
	  return NULL;
     }

     /* ok, they're in. */
     ret = read_user(typed);
     printw("Welcome, %s.\n",ret->handle);
     
     return ret;
     
}

/* central control.  get the user to where they need to be. */

int menu(void){
     
     char buffer[99];
     int opt;
     BBS_Menu *cur;
     int y;
     MSG_Area *msg_base;
     FILE_Area *file_base;
     int x;

     /* bring in the current menu. */
     
     if ((cur = read_menu(this_node->current_menu)) == NULL) {
	  bbserr("Menu load failed.\n");
	  return -1;
     }

     /* if we're in a message area, read it in. */
     if(this_node->area[0] != 0) 
	  msg_base = read_area(this_node->area);
     else
	  msg_base = NULL;

     /* read in any appropriate file areas */
     if(this_node->filearea[0] != 0) {
	  file_base = file_area(this_node->filearea);
     }
     else
	  file_base = NULL;


     /* tell the server that this guy's just sitting idle. */

     sprintf(this_node->what, "%20s %50s",this_node->current_user->handle,"Idle");
     update_node(this_node);

     /* iterate in an ugly manner to display the menu selections.
	should probably be rewritten with the ncurses menu lib stuff
	sometime. */

     x = -1;
     
/* oh god.   second kludge, same as the first....   well, not quite, but we
   do kludge again, as there's a fuckedup menu entry that read_menu didn't
   get, despite getting the rest alright.  damn. */
    
     move(LINES-1, 0);

     color_set(COLOR_BLUE, NULL);
     attron(A_BOLD);
     do {
	  x++;
	  clrtoeol();
	  printw("%c", cur[x].key);
	  attroff(A_BOLD);
	  addch(')');
	  attron(A_BOLD);
	  printw("%-37s", cur[x].label);
	  /* did we finish? */
	  if(cur[x].done == 1) {
	       addch('\n');
	       break;
	  }

	  x++;
	  printw("%c", cur[x].key);
	  attroff(A_BOLD);
	  addch(')');
	  attron(A_BOLD);
	  printw("%-37s\n",cur[x].label);
     } while(cur[x].done != 1);

     attroff(A_BOLD);
     color_set(COLOR_GREEN, NULL);
     
     /* are they in a message area?  tell them stuff about it if so. */
     if(msg_base != NULL) 
	  printw("Message Area: %s -- %s [%d - %d](%d)\n", msg_base->area_name,
		 msg_base->desc, msg_base->first, msg_base->last,
		 this_node->cur);

     /* print information for an applicable file area */
     if(file_base != NULL)
	  printw("File Area: %s -- %s\n", file_base->name, file_base->desc);

     /* go into the keystroke wait loop.  check their remaining time
	every so often so that we're sure that they're not out of
	time. */

     opt = ERR;

     while ((opt == ERR) || (opt == 10)) {

	  /* time check */
	  this_node->current_user->remaining = this_node->current_user->remaining - (time(NULL) - last_checked);
	  last_checked = time(NULL);
	  move(LINES-1, 0);
	  prompt("(%d mins left): ",this_node->current_user->remaining/60);

	  /* out of time? */
	  if(this_node->current_user->remaining < 1) {
	       bbserr("Sorry, you're out of time for today.  Please connect\n");
	       bbserr("again tomorrow.  Thanks for calling.\n");
	       return -1;
	  }
	  
	  
	  /* let them select, and figure out what menu entry goes with that
	     selection. */
	  
/*	  cbreak(); */
	  halfdelay(250);
	  opt = getch();


     }
     addch('\n');

     color_set(COLOR_WHITE, NULL);

     /* cycle through the menu entries and see which one they picked */
     x = -1;
     
     do {
	  x++;
	  if(opt == cur[x].key)
	       break;
     } while(cur[x].done != 1);

     /* hmmm...  they picked something that doesn't exist. */
     if(opt != cur[x].key) {
	  bbserr("Invalid menu option.\n");
	  free(cur);
	  return 0;
     }

     /* tell the server what they're doing now... */

     sprintf(this_node->what, "%20s %50s",this_node->current_user->handle, cur[x].label);
     update_node(this_node);

     /* and, DTRT */

     if(this_node->current_user->access >= cur[x].access) {
	  switch(cur[x].type) {
	  case SUB_MENU:
	       for(y = 0 ; y < 40 ; y++)
		    this_node->current_menu[y] = 0;
	       strcpy(this_node->current_menu,cur[x].action);
	       break;
	  case GOODBYE:
	       return 0xbeef;
	  case WHO:
	       who();
	       addch('\n');
	       break;
	  case INTER_MSG:
	       talk_to_node();
	       break;
	  case INTER_AVAIL:
	       if(this_node->avail == 2) {
		    addstr("You are no longer available for messages.\n");
		    this_node->avail = 1;
		    update_node(this_node);
	       }
	       else {
		    addstr("You are now available for messages.\n");
		    this_node->avail = 2;
		    update_node(this_node);
	       }
	       break;
	  case INTER_CHAT:
	       want_chat();
	       break;
	  case BULLETIN:
	       display_file(cur[x].action);
	       key_pause();
	       break;
	  case SETTINGS:
	       mod_user(this_node->current_user);
	       break;
	  case MSG_START:
	       if(enter_messages() != -1) {
               for(y = 0 ; y < 40 ; y++)
                    this_node->current_menu[y] = 0;
               strcpy(this_node->current_menu, cur[x].action);
	       }
	       break;
	  case MSG_END:
	       this_node->area[0] = 0;
               for(y = 0 ; y < 40 ; y++)
                    this_node->current_menu[y] = 0;
               strcpy(this_node->current_menu, cur[x].action);
	       break;
	       
	  case MSG_AREAS:
	       list_areas();
	       break;
	  case MSG_LIST:
	       list_messages();
	       break;
	  case MSG_VIEW:
	  {
	       MSG_Ref lookup;
	       strcpy(lookup.name, this_node->area);
	       lookup.msg = 0;

	       msg_view(&lookup);
	       break;
	  }
	  case MSG_POST:
	       compose_message(NULL, NULL);
	       break;
	  case MSG_DEL:
	       erase_message(0);
	       break;
	  case MSG_SCAN:
	       check_messages();
	       break;
	  case MSG_CHAREA:
	       prompt("Area name: ");
	       scanw("%s", buffer);
	       change_area(buffer);
	       break;
	  case MSG_REPLY: 
	  {
	       MSG_Ref lookup;
	       strcpy(lookup.name, this_node->area);
	       lookup.msg = this_node->cur;

	       compose_message(NULL, &lookup);
	       break;
	  }
	  case MSG_NEXTAREA:
	       next_area();
	       break;
	  case MSG_PREVAREA:
	       prev_area();
	       break;
	  case DOORSYS:
	  case DORINFO:
	  case DORINFOREM:
	  case AHPAH:
	  case CHAIN_TXT:
	       this_node->avail = this_node->avail-2;
	       update_node(this_node);
	       spawn_door(&(cur[x]));
	       this_node->avail =  this_node->avail+2;
	       update_node(this_node);
	       break;
	  case FILE_START:
	       if(enter_files() != -1)
		    strcpy(this_node->current_menu, cur[x].action);
	       break;
	  case FILE_END:
	       this_node->filearea[0] = 0;
	       strcpy(this_node->current_menu, cur[x].action);
	       break;
	  case FILE_AREAS:
	       list_fileareas();
	       break;
	  case FILE_LIST:
	       list_files();
	       break;
	  case FILE_GET:
	       download();
	       break;
	  case FILE_SEND:
	       upload();
	       break;
	  case FILE_CHAREA:
	  {
	       char newarea[20];
	       FILE_Area *area;

	       printw("Change to: ");
	       scanw("%s", newarea);
	       if((area = file_area(newarea)) == NULL) {
		    bbserr("Invalid area\n");
		    break;
	       }
	       strcpy(this_node->filearea, area->name);
	       free(area);
	       break;
	  }
	  case FILE_NEXTAREA:
	  {
	       FILE_Area *next;

	       if((next = file_nextarea(file_base)) == NULL) {
		    bbserr("No more file areas\n");
		    break;
	       }

	       strcpy(this_node->filearea, next->name);
	       free(next);
	       break;
	  }
	  case FILE_PREVAREA:
	  {
	       FILE_Area *prev;
	       
	       if((prev = file_prevarea(file_base)) == NULL) {
		    bbserr("No previous file area\n");
		    break;
	       }

	       strcpy(this_node->filearea, prev->name);
	       free(prev);
	       break;
	  }
	  	  
	  default:
	       bbserr("This menu option has not yet been configured.\n");
	  }
     } else {
	  /* access too low... */
	  bbserr("Insufficient access for that menu option.\n");
     }
     
     /* and clean up after ourselves like a good little boy. */

     free(cur);
     free(msg_base);

     return 0;
     
}

/* in theory, this makes sure our exit condition is a happy one..  in
   practice...  */

void clean_death(int sig) {
     
     char logbuf[80];

/*     endwin(); */

     sprintf(logbuf, "node %d dying on %d\n", my_node, sig);
     log(logbuf);

     clear_node(my_node); 
     if(this_node->current_user != NULL) 
	  update_user(this_node->current_user);

     sprintf(logbuf,"Node %d disconnected.\n",my_node);
     log(logbuf);
     sleep(1);
     /* kludge to see if we've been run from a thread.  the wrapper
	program will explicitly make primary NULL, but it will be
	defined if something else launched us. */

     return;

}

/* note: continue documentation from here. */

void new_user(void){

     BBS_User *user;
     char *mailer_comm;
     FILE *mailer;
     char pass[30];
     char *pass2;
     char buffer[80];

     user = (BBS_User *)calloc(1,sizeof(BBS_User));

     printw("Welcome to %s.\n",config->system_name);
     addstr("First, We'll have you enter some brief information about yourself.  This\n");
     addstr("information will be kept confidential, with the obvious exception of your\n");
     addstr("handle, which everyone can see.\n\n");

     do {
	  user->handle[0] = 0;
	  addstr("Please select a handle (max. 20 chars): ");
	  scanw("%s",user->handle);
	  /* LEAK */
	  if(read_user(user->handle) != NULL) {
	       addstr("Sorry, that name is already in use.  Please pick another.\n");
	       user->handle[0] = 0; }
     } while(user->handle[0] == 0);
     
     addstr("First name: ");
     scanw("%s",user->first_name);
     addstr("Last name: ");
     scanw("%s",user->last_name);
     addstr("Telephone #: ");
     scanw("%s",user->phone);
     addstr("Make sure this address is correct -- it's used for mailing your password!\n");
     do {
	  addstr("E-mail address (1 user per address!): ");
	  scanw("%s",user->e_mail);
	  e_mail_scan(user); 
     } while(user->e_mail[0] == 0);

     addstr("Address (line 1): ");
     scanw("%[^\n]\n",user->address[0]);
     addstr("Address (line 2): ");
     scanw("%[^\n]\n",user->address[1]);
     addstr("City: ");
     scanw("%s",user->city);
     addstr("State/Province/whatever: ");
     scanw("%s",user->state);
     addstr("Country: ");
     scanw("%s",user->country);

     user->access = 3600;
     user->last_login = time(NULL);
     user->remaining = 3600;
     sprintf(pass,"%d%s",getpid(),user->handle);
     pass2 = (char *)crypt(pass, pass); /* shouldn't crypt BE prototyped? */
     memmove((char *)user->crypt_pass, pass2, strlen(pass2));
     
     addstr("Wow!  You're finished.   You're temporary password will be mailed to your\n");
     addstr("e-mail address immediately (assuming there's no lag on the recieving end.)\n");
     addstr("Once you login with your password, you will be a validated user.\n");

     update_user(user);
     sprintf(buffer, "Created new user %s\n",user->handle);
     log(buffer);

     sprintf(mailer_comm,"/usr/lib/sendmail %s",user->e_mail);

     if((mailer = popen(mailer_comm,"w")) == NULL) {
	  printw("Sorry, the local mailer died.  E-mail %s to have", config->e_mail);
	  printw("your password manually mailed to you");
	  return;
     }

     fprintf(mailer,"From: %s <%s>\n",config->sysop, config->e_mail);
     fprintf(mailer,"To: %s %s <%s>\n",user->first_name, user->last_name,
	     user->e_mail);
     fprintf(mailer,"Subject: New %s account\n\n",config->system_name);
     fprintf(mailer,"Thank you for connecting!\n");
     fprintf(mailer,"Your new password is: %s\n",pass);
     fprintf(mailer,"(though only the first 8 characters are significant)\n");
     fprintf(mailer,"\n.\n");
     pclose(mailer);

     sleep(10);

return;
}

void mod_user(BBS_User *cur_user) {

     MENU *m;
     int quit;
     ITEM *item[10];
     char salt[3];
    
     item[0] = new_item("Phone Number","");
     item[1] = new_item("Address (line 1)","");
     item[2] = new_item("Address (line 2)","");
     item[3] = new_item("City","");
     item[4] = new_item("State","");
     item[5] = new_item("Zip code","");
     item[6] = new_item("Country","");
     item[7] = new_item("Password","");
     item[8] = new_item("Quit","");
     item[9] = NULL;
     
     m = new_menu(item);
     
     quit = 0;
     while(quit == 0) {
	  ITEM *current;
     
     clear();

     mvprintw(1, 30, "Handle: %s", cur_user->handle);
     mvprintw(2, 30, "First name: %s", cur_user->first_name);
     mvprintw(3, 30, "Last name: %s", cur_user->last_name);
     mvprintw(4, 30, "Phone: %s", cur_user->phone);
     mvprintw(5, 30, "Address (line 1): %s", cur_user->address[0]);
     mvprintw(6, 30, "Address (line 2): %s", cur_user->address[1]);
     mvprintw(7, 30, "City: %s", cur_user->city);
     mvprintw(8, 30, "State: %s", cur_user->state);
     mvprintw(9, 30, "Zip: %s", cur_user->zip);
     mvprintw(10, 30, "Country: %s", cur_user->country);

     mvprintw(12, 30, "E-mail: %s", cur_user->e_mail);
	  
	  menu_sel(m);
	  
	  current = current_item(m);
	  
	  if (item[0] == current)
	       strcpy(cur_user->phone, prompt_box("Phone number:"));
	  else if (item[1] == current)
	       strcpy(cur_user->address[0], prompt_box("Address (line 1):"));
	  else if (item[2] == current)
	       strcpy(cur_user->address[1], prompt_box("Address (line 2):"));
	  else if (item[3] == current)
	       strcpy(cur_user->city, prompt_box("City:"));
	  else if (item[4] == current) 
	       strcpy(cur_user->state, prompt_box("State:"));
	  else if (item[5] == current)
	       strcpy(cur_user->zip, prompt_box("Zip Code:"));
	  else if (item[6] == current)
	       strcpy(cur_user->country, prompt_box("Country:"));
	  else if (item[7] == current) {
	       salt[0] = 65 + (getpid() % 26);
               salt[1] = 65 + (time(NULL) % 26);
	       salt[2] = 0;
	       strcpy(cur_user->crypt_pass,
		      (char *)crypt(prompt_box("Password:"), salt));
	  }
	  else if (item[8] == current)
	       quit = 1;
	  else
	       continue;
	  
     }
     
     quit = 0;
     while(quit < 9) {
	  free_item(item[quit]);
	  quit++;
     }

     update_user(cur_user);
     
     free_menu(m);
     
}


void server_int(int sig){
     int db_sock, len;
     char *smallbuf;
     int node_req, port;
     DB_Req req;
     BBS_Inc message;
     int size;

     signal(SIGUSR2, server_int);

     req.req = INC_PICKUP;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return;

     addstr("\nIncoming transmission...\n");

     if(db_reply(db_sock) == -1) {
	  bbserr("Transmission died on the way in.\n");
	  close(db_sock);
	  return;
     }

     read(db_sock, &message, sizeof(BBS_Inc));
     shutdown(db_sock, 2);
     close(db_sock);

     switch(atoi(message.type)) {
     case 0:
	  /* sweeping the nodes... eventually announce our idle time
	     here. */
	  return;
     case 1:
	  /* midnight maintnence occured...  reset our user's time. */
	  this_node->current_user->remaining = this_node->current_user->access;
	  return;
     case 2:
	  /* server wants us to print a string to the user's
	     screen. */
	  attron(A_REVERSE);
	  mvprintw(LINES-1,0,"%s\n",message.msg);
	  beep();
	  attroff(A_REVERSE);
	  return;
	  
     case 3:
	  /* inbound chat request */
	  sscanf(message.msg, "%d %d", &node_req, &port);
	  inbound_chat(node_req, port);
	  return;
	  
     default:
	  bbserr("Some kind of garbage just came from our server. odd.\n");
	  log("Garbage server request!\n");
	  return;

     }
}


void db_shutdown(int sig) {


     addstr("\n\n");
     addstr("Shutdown ordered by server.  All nodes are now being cleared.");
     addstr("\n\n");

     endwin();
     
     exit(sig);

}
