#include <bbsglob.h>
#include <bbsdefs.h>
#include <db_iface.h>
#include <nodelist.h>
#include <messages.h>
#include <fileareas.h>

#define DEBUG

/* stuff used only internally by the db server */

/* request handling */
void new_request(DB_Req *req, int net);
int key_check(DB_Req *req);

/* user-related stuff */
int do_login(BBS_Login *ret, char *handle, char *password);
int db_user_read(BBS_User *info, char *handle);
int db_user_write(BBS_User *info);
int db_user_clear(char *handle);
BBS_User *db_user_list(int *len);
int db_user_mail(char *addr);

/* node data stuff */
int db_node_read(BBS_Node *info, int node);
int db_node_write(BBS_Node *info);
int db_node_clear(int node);

/* message crap */
int db_nodelist_lookup(tAddress *lookup, tConnect *info);
int db_point_read(int *info, char *index);
int db_point_write(int info, char *index);
int db_point_clear(char *index);
int db_msg_read_head(MSG_Head *info, MSG_Ref *lookup);
char *db_msg_read(int *len, MSG_Ref *lookup);
int db_msg_write(char *info, int size, MSG_Ref *where);
int db_msg_clear(MSG_Ref *lookup);
int db_msg_scan_usr(int **msgs, int *len, char *handle, MSG_Ref *where);
int db_msg_toggle_trans(MSG_Ref *lookup);

/* area (and net) crap */
int db_msg_area(MSG_Area *info, char *name);
int db_msg_arealist(MSG_Area **info, int *len);
int db_msg_area_next(MSG_Area *info, MSG_Area *next);
int db_msg_area_prev(MSG_Area *info, MSG_Area *prev);
int db_msg_net(MSG_Net *info, char *name);
int db_msg_netlist(MSG_Net **info, int *len);

/* menu */
int db_menu_read(BBS_Menu **menus, int *len, char *name);

/* log */
int db_log(int node, char *message);

/* inter-node comms */
int db_inc_send(BBS_Inc *info);
int db_inc_pickup(BBS_Inc *info, int node);

/* file areas */
int db_file_area(FILE_Area *info, char *name);
int db_file_info(FILE_Info *info);
FILE_Info *db_file_list(char *area, int *len);
int db_file_add(FILE_Info *info);
int db_file_clear(FILE_Info *info);
int db_file_protos(FILE_Proto **info, int *len);


/* internal data manipulations */
int reload_config(void);
int midnight(void);
void read_msg_config(void);
void read_net_config(void);
void scan_msg_area(MSG_Area *area);
int renumber(MSG_Area *area, BBS_User *users, int total_users);
int read_file_config(void);

/* structures to hold node, area, and net data in a linked list */

struct ll_nodes {
     BBS_Node data;
     struct ll_nodes *next;
};

struct ll_areas {
     MSG_Area data;
     struct ll_areas *next;
};

struct ll_nets {
     MSG_Net data;
     struct ll_nets *next;
};

struct ll_inc {
     BBS_Inc data;
     struct ll_inc *next;
};

struct ll_files {
     FILE_Area data;
     struct ll_files *next;
};

struct ll_protos {
     FILE_Proto data;
     struct ll_protos *next;
};
