/* deal with the db server's side of the file accesses, which are the
   ones which actually hit the raw files, and stuff. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>

#include "db.h"

#define DEBUG

/* crypt() is fucked up in the headers. */

extern char *crypt(const char *key, const char *salt);

extern BBS_Config *config;
extern struct ll_nodes *node_config;
extern struct ll_inc *inc_list;

/* mutex-related calles removed.  I suppose we'll hack them back in
   when we decide to thread coherently */

/* open_user_db assumes that its caller has already played the happy
   mutex game.  So do the other various openers. */

DBM *open_user_db() {

     char bleh[80];
     static DBM *ret;

     strcpy(bleh, config->base_dir);
     strcat(bleh, "/db/users.db");
     
     ret = dbm_open(bleh, O_RDWR | O_CREAT, S_IREAD | S_IWRITE);
	       
     if(ret == (DBM *)0) {
	  fprintf(stderr,"Couldn't open db: %s\n",bleh);
	  exit(2);
     }

     return ret;

}


int db_user_read(BBS_User *info, char *handle) {

     DBM *db;
     datum key;
     datum data;
     int ret;

     db = open_user_db();
     key.dptr = str_up(handle);
     key.dsize = strlen(handle);
     data = dbm_fetch(db, key);
     
     if (data.dptr != NULL) {
          memcpy(info, data.dptr, data.dsize);
	  ret = SUCCESS;
     }
     else
	  ret = FAILURE; 

     dbm_close(db);
     
return ret;

}

/* unimplemented */
BBS_User *db_user_list(int *len) {
     BBS_User *ret;
     DBM *db;
     datum key;
     datum data;
     int x = 0;

     /* now, I can't think of a way not to have to count this twice,
	so we do.  */
     (*len) = 0;
     
     if((db = open_user_db()) == NULL) 
	  return NULL;
     
     key = dbm_firstkey(db);
     do {
	  (*len)++;
	  key = dbm_nextkey(db);
     } while(key.dptr != NULL);

     /* alright, counted */
     ret = (BBS_User *)malloc(sizeof(BBS_User) * (*len));
     key = dbm_firstkey(db);
     do {
	  data = dbm_fetch(db, key);
	  memcpy(&(ret[x]), data.dptr, sizeof(BBS_User));
	  x++;
	  key = dbm_nextkey(db);
     } while(key.dptr != NULL);

     /* alright, so we filled the shit we allocated */
     dbm_close(db);
     
     return ret;

}

int db_user_write(BBS_User *user) 
{

     DBM *db;
     datum key;
     datum data;
     int ret;

     db = open_user_db();
     key.dptr = str_up(user->handle);
     key.dsize = strlen(user->handle);
     
     data.dptr = (void *)user;
     data.dsize = sizeof(BBS_User);

     ret = dbm_store(db, key, data, DBM_REPLACE);

     dbm_close(db);

     return ret;

}
	       
		    
int db_user_clear(char *handle)
{

     DBM *db;
     datum key;
     int ret;

     db = open_user_db();
     key.dptr = str_up(handle);
     key.dsize = strlen(handle);

     ret = dbm_delete(db, key);

     dbm_close(db);

     return ret;
}

/* scan for duplicate e-mail addresses for new-user creation. */
int db_user_mail(char *addr) {
    
     datum db_read;
     BBS_User *read;

     char buffer[80];
     DBM *ret;

     ret = open_user_db();

     db_read = dbm_firstkey(ret);

     /* iterate through the user records until the end or a match */
     do {
	  db_read = dbm_fetch(ret, db_read);
	  read = (BBS_User *)db_read.dptr;
	  if(!strcasecmp(read->e_mail, addr)) {
	       sprintf(buffer, "Duplicate e-mail attempt on %s.\n", addr);
	       db_log(255, buffer);
	       free(db_read.dptr);
	       dbm_close(ret);
	       return FAILURE; 
	  }

	  db_read = dbm_nextkey(ret);
     } while (db_read.dptr != NULL);
     
     dbm_close(ret);

     return SUCCESS;

}

int db_node_read(BBS_Node *info, int node) {

     struct ll_nodes *cur;
     int ret;

     cur = node_config;
     
     while(cur != NULL) {
	  if(cur->data.node == node)
	       break;
	  else
	       cur = cur->next;
     }
	  
     if (cur != NULL) {
	  memcpy(info, &(cur->data), sizeof(BBS_Node));
	  ret = SUCCESS;
     }
     else
	  ret = FAILURE;

return ret;

}

int db_node_write(BBS_Node *node)
{

     int ret;
     struct ll_nodes *cur;

     cur = node_config;

     while(cur != NULL) {
	  if(node->node == cur->data.node) {
	       /* simple replacement of existing node, we can just
		  copy over old one */

	       memcpy(&(cur->data), node, sizeof(BBS_Node));
	       ret = SUCCESS;
	       break;
	  } else 
	       cur = cur->next;
     }
     
     /* this node's not in the list yet, so pop it onto the top of
	it. */

     if(cur == NULL) {
	  static struct ll_nodes *add;
	  
	  add = (struct ll_nodes *)malloc(sizeof(struct ll_nodes));
	  memcpy(&(add->data), node, sizeof(BBS_Node));
	  add->next = node_config;
	  node_config = add;
	  ret = SUCCESS;
     }	  

     return ret;

}


int db_node_clear(int node)
{
     int ret;
     struct ll_nodes *cur;
     struct ll_nodes *prev;
     struct ll_inc *cur_msg;
     struct ll_inc *prev_msg;

     ret = SUCCESS;

     cur = node_config;
     prev = NULL;

     /* search the linked list for the current node. */
     while(cur != NULL) {
	  if (node == cur->data.node)
	       break;
	  else {
	       prev = cur;
	       cur = cur->next;
	  }
     }


     /* if cur reached NULL, we didn't find the node in the list
	anywhere, so fail. */

     if(cur == NULL) 
	  return FAILURE;

     /* our node was the first one.  How precious. */
     if(prev == NULL) 
	  node_config = cur->next;
     else
	  /* or not, deal with it. */
	  prev->next = cur->next;
     
     free(cur);

     /* now, get rid of any pending messages they might have had. */
     prev_msg = NULL;
     cur_msg = inc_list;
     
     while(cur_msg != NULL) {
	  if(cur_msg->data.dest_node == node) {
	       if(prev_msg == NULL) 
		    inc_list = cur_msg->next;
	       else
		    prev_msg->next = cur_msg->next;
	       free(cur_msg);
	  } else {
	       prev_msg = cur_msg;
	       cur_msg = cur_msg->next;
	  }
     }
 
     return ret;
}

int do_login(BBS_Login *info, char *handle, char *password) {

     BBS_Node cur;
     BBS_User user;
     char salt[2];
     char *crpass;
     int ret;
     
     info->node = 1;

     while((db_node_read(&cur, ret) != FAILURE) && (info->node <= 
						    config->nodes)) {
	  info->node++;
     }
     
/* return a -1, special to indicate that there's not a node for us. */
     if(info->node > config->nodes) /* we're over the node cap. */
	  return -1;

     if(db_user_read(&user, handle) == FAILURE) 
	  return FAILURE;
     
     salt[0] = user.crypt_pass[0];
     salt[1] = user.crypt_pass[1];
     crpass = (char *)crypt(password, salt);

     if(!strncmp(user.crypt_pass, crpass, 13))
	  return SUCCESS; 
     else 
 	  return FAILURE;

}

