
/* here's the stuff that deals with messages...  we're shooting to do
   this in fido style, as we want to be able to deal with fidonet
   messages without converting them to unix-style mail and news --
   that would be stupid, because we're not using the unix level user
   information, and trying to mesh it all would, basically, suck. */


/* fts-0001 message header format. */
/* we're going to read/write this assuming a little endian machine for
   now.  I'll put in some big ending conversions eventually. */

typedef struct _MSG_Head {
     char from[36];
     char to[36];
     char subject[72];
     char date[20];
     u_int16_t timesread;
     u_int16_t destnode;
     u_int16_t orignode;
     u_int16_t cost;
     u_int16_t orignet;
     u_int16_t destnet;
     u_int16_t destzone;
     u_int16_t origzone;
     u_int16_t destpoint;
     u_int16_t origpoint;
     u_int16_t replyto;
     u_int16_t attribute;
     u_int16_t nextreply;
    
} MSG_Head;

/* #defines for the attribute bits. */

#define MSG_PRIV 0x0001
#define MSG_CRASH 0x0002
#define MSG_RECD 0x0004
#define MSG_SENT 0x0008
#define MSG_ATTACH 0x0010
#define MSG_TRANSIT 0x0020
#define MSG_ORPHAN 0x0040
#define MSG_KILLSENT 0x0080
#define MSG_LOCAL 0x0100
#define MSG_HOLD 0x0200
#define MSG_UNUSED 0x0400
#define MSG_FREQ 0x0800
#define MSG_RCPTREQ 0x1000
#define MSG_ISRCPT 0x2000
#define MSG_AUDIT 0x4000
#define MSG_UPDREQ 0x8000

#define MSG_CONT 0x01

/* the header given to a message while it lives in the packet.   God,
   this makes me want to really, really curse ancient modems for
   forcing these byte savings upon us now. */

typedef struct _MSG_Head_Pack {
     u_int16_t magic; /* 0x02 0x00 */
     u_int16_t orignode;
     u_int16_t destnode;
     u_int16_t orignet;
     u_int16_t destnet;
     u_int16_t attribute;
     u_int16_t cost;
     char date[20];
     /* the following are variable-length strings...  allocate for the
	max, naturally, but remember to only do null-termination. */
     char to[36];
     char from[36];
     char subject[72];
} MSG_Head_Pack;

/* actual header for a packet. */
typedef struct _MSG_Pack {
     u_int16_t orignode;
     u_int16_t destnode;
     u_int16_t year;
     u_int16_t month;
     u_int16_t day;
     u_int16_t hour;
     u_int16_t minute;
     u_int16_t second;
     u_int16_t baud;
     u_int16_t magic1; /* 0x02 0x00 */
     u_int16_t orignet;
     u_int16_t destnet;
     u_int8_t prod_lo;
     u_int8_t rev_lo;
     u_int8_t password[8]; /* null padded */
     u_int16_t qorigzone;
     u_int16_t qdestzone;
     u_int16_t auxnet;
     u_int16_t cap_valid;
     u_int8_t prod_hi;
     u_int8_t rev_hi;
     u_int16_t cap_word;
     u_int16_t origzone;
     u_int16_t destzone;
     u_int16_t origpoint;
     u_int16_t destpoint;
     u_int8_t proddata[4];
     
     /* remainder consists of a bunch of packed messages */
} MSG_Pack;


/* and the configs for the local message areas. */

typedef struct _MSG_Area {
     char path[36];
     char area_name[20];
     char desc[50];
     int first;
     int last;
     int type; /* ignored.  only .msg supported. */
     int network_num; /* 0 - local, 1+ FTN nets */
     int expire; /* in days */
     int private; 
     int real;
     unsigned int access;
     int done;
} MSG_Area;

typedef struct _MSG_Net {
     int netid;
     char name[20];
     char desc[80];
     char addr_str[80];
     u_int16_t zone;
     u_int16_t net;
     u_int16_t node;
     u_int16_t point;
     char netmail[80];
     char outgoing[80];
     char host[80];
     int done;
} MSG_Net;

/* used for referring to a particular message out there. */

typedef struct _MSG_Ref {
     int msg; /* message number */
     char name[20]; /* area name */
} MSG_Ref;

