/* cli_nodes.c -- generic node operations. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <curses.h>
#include <unistd.h>

#include <client.h>

#define DEBUG

extern BBS_Config *config;
extern int my_node;

BBS_Node *read_node(int node) {
     
     int db_sock, len;
     BBS_Node *ret;

     DB_Req req;
     int size;

     req.req = NODE_READ;
     req.datasize = sizeof(int);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, &node, sizeof(int));

     if(db_reply(db_sock) == -1)
	  return NULL;

     ret = (BBS_Node *)malloc(sizeof(BBS_Node));

     len = read(db_sock, ret, sizeof(BBS_Node));
     
     if(len < sizeof(BBS_Node)) {
	  shutdown(db_sock,2);
	  close(db_sock);
	  free(ret);
	  return NULL;
     }
     
     shutdown(db_sock,2);
     close(db_sock);
     return ret;
     
}

int update_node(BBS_Node *node) 
{

     int db_sock;
     DB_Req req;
     int size;

     req.req = NODE_WRITE;
     req.datasize = sizeof(BBS_Node);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, node, sizeof(BBS_Node));

     if(db_reply(db_sock) == -1) {
	  shutdown(db_sock, 2);
	  close(db_sock);
          return -1;
     }

     shutdown(db_sock, 2);
     close(db_sock);

     return 0;
     
}

int clear_node(int node)
{
     int db_sock;
     DB_Req req;
     int size;

     req.req = NODE_CLEAR;
     req.datasize = sizeof(int);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;
     
     write(db_sock, &node, sizeof(int));
     if(db_reply(db_sock) == -1) {
	  shutdown(db_sock, 2);
	  close(db_sock);
          return -1;
     }

     shutdown(db_sock,2);
     close(db_sock);
     return 0;
}
	       
int login_node(char *login, char *password) {

     int db_sock;
     BBS_Login info;
     DB_Req req;
     int size;

     req.req = LOGIN;
     req.datasize = 40;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, login, 20);
     write(db_sock, password, 20);

     if(db_reply(db_sock) == -1) {
	  close(db_sock);
	  return -1;
     }

     read(db_sock, &info, sizeof(BBS_Login));

     close(db_sock);
     return info.node;
     
}

