/* cli_users.c -- contains client library functions for dealing with
   the users in some ways, not related to their presence on a given
   node. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <curses.h>
#include <unistd.h>

#include "client.h"

#define DEBUG

extern int my_node;
extern BBS_Config *config;

/* as with the messages, we need to force a stall so that records don't
   end up short when we're reading a bunch of them.  I find it odd
   that this happens, we may need to rethink our read strategy. */
#define RECV_STALL 12500 /* 1/80 second works here */


BBS_User *read_user(char *handle) {
     
     int db_sock, len;
     static BBS_User *ret;
     DB_Req req;
     int size;

     req.req = USER_READ;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;     

     if((db_sock = db_conn(&req)) == -1) 
	  return NULL;
     

     write(db_sock, handle, 20);
     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
	  return NULL;
     }

     ret = (BBS_User *)malloc(sizeof(BBS_User));
     
     if((len = read(db_sock, ret, size)) < size) {
	  free(ret);
	  close(db_sock);
	  return NULL;
     }

     shutdown(db_sock, 2);
     close(db_sock);
     return ret;
     
}

int update_user(BBS_User *user) 
{

     int db_sock;
     
     DB_Req req;
     int size;

     req.req = USER_WRITE;
     req.datasize = sizeof(BBS_User);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, user, sizeof(BBS_User));

     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
	  return -1;
     }	

     close(db_sock);
     return 0;
     
}
	       
		    
int clear_user(char *handle)
{
     int db_sock;

     DB_Req req;
     int size;

     req.req = USER_CLEAR;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, handle, 20);

     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
	  return -1;
     }

     close(db_sock);
     return 0;
}

void e_mail_scan(BBS_User *user) {
     
     int db_sock;

     DB_Req req;
     int size;

     req.req = USER_EMAIL_SCAN;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, user->e_mail, 60);

     if(db_reply(db_sock) == -1) {
	  user->e_mail[0] = 0;
	  close(db_sock);
          return;
     }

     shutdown(db_sock, 2);
     close(db_sock);
     return;
}

BBS_User *read_userlist(int *len) {

     BBS_User *ret;
     
     int db_sock;

     DB_Req req;
     int size, x;

     req.req = USER_LIST;
     req.datasize = 0;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
	  return NULL;
     }

     (*len) = size / sizeof(BBS_User);
     ret = (BBS_User *)malloc(size);

     big_read(db_sock, ret, size);

     close(db_sock);
     return ret;
     
}
