
/* packer.c -- handles making packets out of messages. */

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <curses.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>

#include <common.h>
#include <client.h>

#include "packer.h"

MSG_Area *areas;
MSG_Net *net;

int my_node = 0;
BBS_Config *config;
int ticks;

char *id_to_name(int name);

/* set up something to tell when there's archiving necessary for a
   given network.  keeps archiving from happening when there's only
   netmail.  Also, set up a second array of strings which contain any
   packet (should never be more than one per net, with only one
   netmail area) which contains netmail, and, as such, shouldn't be
   archived. */
            

main(int ac, char *av[]){

     MSG_Area *cur;
     int netcount;
     int x;

     config = (BBS_Config *)malloc(sizeof(BBS_Config));
     read_system_config();

     net = all_nets();
     x = 0;


     /* count how many nets we have so that we can malloc an
	appropriate amount of ram. */
     /* granted, though, this whole routine assumes that your netid's
	are: 1) 0-based.
	2) contiguous.
	This, all things considered, is a shitty set of assumptions.
     */
     
     if((areas = all_areas()) == NULL) {
	  printf("Failed to read area list from server.\n");
	  exit(0);
     }

     x = 0;
     do {
	  scan_area2(&(areas[x]));
	  x++;
     } while(areas[x].done != 1);

     x = -1;
     do {
	  x++;
	  if(net[x].netid == 0) 
	       continue;

	  archive(&(net[x]));
     } while(net[x].done != 1);

     /* fuck cleaning up the ram, I think we'll just terminate.  :) */

     exit(0);
     
}


void scan_area2(MSG_Area *area) {

     struct tm *out_time;
     int x;
     int tot;
     int trash1, trash2;
     char buffer[80];
     MSG_Pack *pack_head;
     MSG_Net *this_net;
     int size;
     int pktfd;
     int pknm;
     char pktname[160];
     time_t dtime;
     int last_scan;
     char *netname;

     netname = id_to_name(area->network_num);
     this_net = read_net(netname);

     free(netname);

     if((this_net == NULL) || (this_net->netid == 0)) {
	  printf("Skipping local group %s\n", area->area_name);
	  if(this_net != NULL)
	       free(this_net);
	  return;
     }

     printf("Scanning %s\n", area->area_name);

     if((pack_head = prep_packet(area)) == NULL) {
          printf("Couldn't toss in %s -- Invalid net config.\n");
          free(this_net);
	  return;
     }

     /* We'll have the tosser act like user oxiduspack, so that it can
	maintain pointers to the last message scanned.  should speed
	things up a good bit. */

     last_scan = read_pointer("oxiduspack", area->area_name);

     tot = (last_scan - area->last);

     if(tot == 0) {
	  free(this_net);
	  free(pack_head);
          return;
     }

     x = last_scan;
     
     pknm = 0;

     dtime = time(NULL);
     out_time = gmtime(&dtime);

     sprintf(pktname, "%s/%s/%.2d%.2d%.2d%.2d.pkt", config->base_dir,
	     this_net->outgoing, out_time->tm_hour,
	     out_time->tm_min, out_time->tm_sec, ticks);
     
     ticks++;
     
     if((pktfd = open(pktname, O_WRONLY | O_CREAT | O_EXCL, S_IREAD |
		      S_IWRITE)) == -1) {
	  perror("failed to vreate packet");
	  free(this_net);
	  free(pack_head);
	  return;
     }
     
     size = write(pktfd, pack_head, sizeof(MSG_Pack));
     printf("wrote %d bytes\n", size);
     
     while (x <= area->last) {

          MSG_Head *head;
          MSG_Head_Pack *phead;
	  MSG_Ref where;
          char *body;
	  char *heads;
	  char *msgid;
	  char *feet;
	  
	  strcpy(where.name, area->area_name);
	  where.msg = x;

	  printf("on message %d\n", x);
          head = (MSG_Head *)malloc(sizeof(MSG_Head));
          body = read_message(&where, head);
          if(body == NULL) {
	       printf("Unable to read message %d\n", x);
               free(head);
	       x++;
               continue;
          }

	  /* if the transit bit is not set, we can assume that this
	     message does not need to go out to the network...  */
          if(!(head->attribute & MSG_TRANSIT)) {
               free(head);
               free(body);
               x++;
	       continue;
          }

	  /* likewise if the sent bit is set. */
	  if((head->attribute & MSG_SENT)) {
	       free(head);
               free(body);
               x++;
               continue;
          }

	  /* toggle off the transit flag */
	  tr_off(&where);
	  
	  msgid = (char *)calloc(sizeof(char),80);
	  
	  sprintf(msgid, "%d:%d/%d.%d %x", this_net->zone,
		  this_net->net, this_net->node, this_net->point,
		  time(NULL)+(ticks * 0x100));
	  
	  phead = (MSG_Head_Pack *)malloc(sizeof(MSG_Head_Pack));

          phead->magic= 0x02;
          phead->orignode = head->orignode;
          phead->orignet = head->orignet;
	  sscanf(this_net->host, "%hd:%hd/%hd.%hd", &trash1,
		 &phead->destnet, &phead->destnode, &trash2);
	  phead->attribute = head->attribute ^ MSG_TRANSIT;
	  phead->cost = 0;
	  strcpy(phead->date, head->date);
	  strcpy(phead->to, head->to);
	  strcpy(phead->from, head->from);
	  strcpy(phead->subject, head->subject);

	  /* mangle msgid, area, int, seen-by, origin into body! */

	  heads = (char *)malloc(1024+strlen(body));
	  feet = (char *)malloc(768);
	  
	  /* add reply tracking! no, the system will do it */
	  if(!strcasecmp(area->area_name, this_net->netmail)) {
	       if(head->destzone != 0) {
	       /* well, this is our netmail area.  make more
		  INTeresting headers. */
	       sprintf(heads, "%cINTL %d:%d/%d.%d %s\r%cMSGID %s\r%cPID: OxidusPack\r", 0x01, 
		       head->destzone, head->destnet, head->destnode,
		       head->destpoint, this_net->addr_str, 0x01,
		       msgid, 0x01);
	       /* phew! */
	       } else {
		    /* bre mangles it's netmail.  how unfortunate. */
               sprintf(heads, "%cMSGID %s\r%cPID: OxidusPack\r", 0x01,
		       this_net->addr_str, 0x01,
                       msgid, 0x01);
	       }
	  } else {
	       sprintf(heads, "AREA:%s\r%cMSGID %s\r%cPID: OxidusPack\r",
		       area->area_name, 0x01, msgid, 0x01);
	  }
	  
	  strcat(heads, body);
	  free(body);
	  heads[strlen(heads)-1] = '\r'; /* strip null term */
	  sprintf(feet, "--- %s\r * Origin: %s (%s)\rSEEN-BY: %d/%d\r"
		  ,VERSION, config->system_name, this_net->addr_str,
		  this_net->net, this_net->node);
	  
	  strcat(heads, feet);
	  /* wow, now it should be fucking ready */

	  size += write(pktfd, phead, sizeof(MSG_Head_Pack) - 144);
	  size += write(pktfd, phead->to, strlen(phead->to)+1);
	  size += write(pktfd, phead->from, strlen(phead->from)+1);
	  size += write(pktfd, phead->subject, strlen(phead->subject)+1);
	  size += write(pktfd, heads, strlen(heads));
	  
	  /* well, the whole fucker should be there. */
	  
	  if(head->attribute & MSG_ATTACH) {
	       char *temp;
	       char dest[120];
	       char src[120];
	       temp = strrchr(head->subject, '\\');
	       if(temp == NULL)
		    temp = head->subject;
	       else
		    temp++;
	       printf("found attachment %s\n", temp);
	       sprintf(dest, "%s/%s/%s", config->base_dir, 
		       this_net->outgoing, str_down(temp));
	       sprintf(src, "%s/messages/%s/%s", config->base_dir, area->path,
		       str_down(temp));
	       printf("copying %s to %s\n", src, dest);
	       simple_copy(src, dest);
	  }

	  x++;
	  free(phead);
	  free(head);
	  free(heads);
     
     }

     /* update the pointer for the packer, do x-1 just to be sure
	nothing gets dropped at any point... */
     if(size == sizeof(MSG_Pack))
	  x++;

     update_pointer("oxiduspack", area->area_name, x-1);
     
     close(pktfd);

     if(size == sizeof(MSG_Pack))
	  /* twas empty.. */
	  unlink(pktname);

     free(this_net);
     free(pack_head);

     return;

}

MSG_Pack *prep_packet(MSG_Area *area){

     MSG_Net *this_net;
     MSG_Pack *ret;
     struct tm *timest;
     time_t dtime;
     char *netname;

     netname = id_to_name(area->network_num);
     this_net = read_net(netname);
     free(netname);

     if(this_net == NULL) {
	  printf("Gross network config error.\n");
	  return NULL;
     }

     ret = (MSG_Pack *)calloc(sizeof(MSG_Pack), 1);
     
     dtime = time(NULL);
     timest = gmtime(&dtime);

     ret->origzone = this_net->zone;
     ret->orignet = this_net->net;
     ret->orignode = this_net->node;
     ret->origpoint = this_net->point;
     sscanf(this_net->host, "%hd:%hd/%hd.%hd", &ret->destzone,
	    &ret->destnet, &ret->destnode, &ret->destpoint);
     
     ret->year = timest->tm_year;
     ret->month = timest->tm_mon;
     ret->day = timest->tm_mday;
     ret->hour = timest->tm_hour;
     ret->minute = timest->tm_min;
     ret->second = timest->tm_sec;
     ret->baud = 9600;
     ret->magic1 = 0x02;
     ret->prod_lo = 0xFF;
     ret->rev_lo = 0xFF;
     ret->qorigzone = ret->origzone;
     ret->qdestzone = ret->destzone;
     ret->prod_hi = 0xFF;
     ret->rev_hi = 0xFF;

     /* and that should do it. */
     
     free(this_net);
     /* free(timest); fufc! */
     return ret;

}

int archive(MSG_Net *net) {

     char *cmdline;
     char *archname;
     struct dirent **packs;
     int pktcount;
     char day[3];
     char *names;
     struct tm *timestr;
     struct stat archst;
     time_t tsecs;
     int destnet, destnode, junk;

     tsecs = time(NULL);

     timestr = gmtime(&tsecs);

     sscanf(net->host, "%hd:%hd/%hd", &junk, &destnet, &destnode);

     switch(timestr->tm_wday) {
     case 0:
	  strcpy(day, "su");
	  break;
     case 1:
	  strcpy(day, "mo");
	  break;
     case 2:
	  strcpy(day, "tu");
	  break;
     case 3:
	  strcpy(day, "we");
	  break;
     case 4:
	  strcpy(day, "th");
	  break;
     case 5:
	  strcpy(day, "fr");
	  break;
     case 6:
	  strcpy(day, "sa");
	  break;
     default:
	  printf("Wow, gmtime() is mangling it's output reeeeel good. Can't archive!\n");
	  return -1;
     }

     if((pktcount = scandir(net->outgoing, &packs, pkt_match, alphasort)) < 1) {
	  printf("Error (or no packets) scanning %s.\n", net->outgoing);
	  return -1;
     }
     
     names = (char *)calloc(strlen(net->outgoing)+15,pktcount);

     while(pktcount > 0) {
	  pktcount--;
	  strcat(names, net->outgoing);
	  strcat(names, "/");
	  strcat(names, packs[pktcount]->d_name);
	  strcat(names, " ");
     }

     archname = (char *)calloc(sizeof(char),128);
     cmdline = (char *)calloc(sizeof(char), 256);

     sprintf(archname, "%s/%s/%.4x%.4x.%s%d",config->base_dir,
             net->outgoing, (short)(net->net-destnet),
	     (short)(net->node-destnode), day, transmits(net));

     sprintf(cmdline, "%s %s %s", "zip ", archname, names);

     system(cmdline);

     if(stat(archname, &archst) != 0) {
	  printf("Couldn't stat %s, creation faliled.\n",archname);
	  return -1;
     }
     
     printf("Packet %s created. (%d bytes)\n", archname, archst.st_size);
     return 0;

}     

     
int transmits(MSG_Net *net) {

     return 0;

}

char *id_to_name(int id) {

     int x = -1;
     char *ret;

     ret = (char *)calloc(1, 20);

     /* cycle looking for the id. */
     do {
	  x++;
	  if(net[x].netid == id) {
	       strcpy(ret, net[x].name);
	       break;
	  }
     } while(net[x].done != 1);
     
     /* did we ever make a copy? */
     if(ret[0] == 0) {
	  free(ret);
	  return NULL;
     }
	  
     return ret;

}

int pkt_match(struct dirent *entry) {

     int ret;

     if((strstr(entry->d_name, ".pkt") != NULL) ||
	(strstr(entry->d_name, ".PKT") != NULL ))
          return 1;
     else
          return 0;

}
