
/* filetool.c -- the first config tool to move to a nicer, menued,
   curses-style setup...  this one was about the most pita to use,
   anyway. */

#include <stdio.h>
#include <ncurses.h>
#include <menu.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>

#include <common.h>
#include <client.h>
#include <disp.h>

int my_node = 0;
BBS_Config *config;
BBS_Node *this_node; /* the display functions want it, be we don't
			define it */

main(int ac, char *av) {

     int quit = 0;
     MENU *m;
     ITEM *items[8];
     char cur_area[20];

     config = calloc(1, sizeof(BBS_Config));
     read_system_config();

     initscr();
     immedok(stdscr, TRUE);
     keypad(stdscr, TRUE);
     scrollok(stdscr, TRUE);

     cbreak();

     items[0] = new_item("List Areas", "");
     items[1] = new_item("List Files", "");
     items[2] = new_item("View File Information", "");
     items[3] = new_item("Add File", "");
     items[4] = new_item("Remove File", "");
     items[5] = new_item("Change Area", "");
     items[6] = new_item("Quit", "");
     items[7] = NULL;

     m = new_menu(items);

     /* we're not in an area yet */
     cur_area[0] = 0;

     while(quit == 0) {
	  ITEM *current;
	  
	  clear();

	  if(cur_area[0] != 0) 
	       mvprintw(LINES-3, 1, "Current area: %s", cur_area);
 
	  menu_sel(m);
	  current = current_item(m);
	  clear();

	  /* don't let them do most stuff if an area hasn't been
	     chosen */
	  if(cur_area[0] == 0)
	       if((current != items[0]) && (current != items[6]) &&
		  (current != items[5])) {
	       printw("You must first choose an area!\n");
	       key_pause();
	       continue;
	  }

	  if(items[0] == current)
	  {
	       FILE_Area *list;
	       int len, x;
	       x = 0;

	       if((list = file_arealist(&len)) == NULL) {
		    printw("Server error, or no areas configured.\n");
		    key_pause();
		    continue;
	       }

	       while(x < len) {
		    printw("%20s %s\n", list[x].name, list[x].path);
		    x++;
	       }

	       free(list);
	       key_pause();

	  }

	  else if(items[1] == current)
	  {
	       int len, x;
	       FILE_Info *files;

	       if((files = file_list(cur_area, &len)) == NULL) {
		    printw("No files, or server error\n");
		    key_pause();
		    continue;
	       }
	       
	       x = 0;
	       while(x < len) {
		    printw("%8d %s\n", files[x].size, files[x].name);
		    x++;
	       }

	       free(files);
	       key_pause();
	  }
	  
	  else if(items[2] == current)
	  {
	       FILE_Info file;
	       
	       strcpy(file.area, cur_area);
	       addstr("name: ");
	       scanw("%s", file.name);
	       
	       if(file_info(&file) == -1) {
		    printw("No such file in that area\n");
		    key_pause();
		    continue;
	       }
	       
	       printw("%8d %s\n", file.size, file.name);
	       addstr(file.desc);
	       addch('\n');

	       free(file.desc);
	       
	       key_pause();
	  }

	  else if(items[3] == current)
	  {
	       FILE_Info new;
	       FILE_Area *area;
	       struct stat statbuf;
	       char desc[512];
	       char fullpath[160];

	       strcpy(new.area, cur_area);
	       
	       area = file_area(new.area);
	       addstr("name: ");
	       scanw("%s", new.name);

	       sprintf(fullpath, "%s/%s/%2", config->base_dir,
		       area->path, new.name);

	       if(stat(fullpath, &statbuf) == -1) {
		    printw("Couldn't stat %s\n", fullpath);
		    free(area);
		    key_pause();
		    continue;
	       }

	       new.size = statbuf.st_size;

	       addstr("description: ");
	       getstr(desc);

	       new.desclen = strlen(desc);
	       new.desc = malloc(new.desclen);
	       memcpy(new.desc, desc, new.desclen);

	       printw("Adding %d length file w/length %d description:\n%s\n",
		      statbuf.st_size, new.desclen, new.desc);


	       if(file_add(&new) != 0) 
		    printw("Add failed\n");
	       else
		    printw("File added.\n");

	       free(new.desc);
	       
	       key_pause();
	  }

	  else if(items[4] == current)
	  {
	       FILE_Info clear;

	       strcpy(clear.area, cur_area);
	       addstr("name: ");
	       scanw("%s", clear.name);
		    
	       if(file_clear(&clear) == -1) 
		    addstr("Invalid area or file\n");
	       else
		    addstr("File cleared\n");

	       key_pause();
	  }
	  else if(items[5] == current) {

	       char *new_area;
	       FILE_Area *temp;

	       new_area = prompt_box("New area:");
	       
	       if((temp = file_area(new_area)) == NULL) {
		    printw("Invalid area!\n");
		    free(new_area);
		    continue;
	       }

	       free(temp);
	       strcpy(cur_area, new_area);
	       free(new_area);
	       	       
	  }

	  else if(items[6] == current)
	       break;

     }

     endwin();
     return 0;
     
}




