#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <nodelist.h>
#include "nl2k_compile.h"
#undef NL2K_DEBUG

/* maintain the starting point for every linked list. */

struct tNLDat_wrap *start_email;
struct tNLDat_wrap *start_ftp;
struct tNLDat_wrap *start_phone;
struct tNLDat_wrap *start_telnet;
struct tNLDat_wrap *start_isdn;
struct tNLDat_owrap *start_other;

void compile_main(char **filenames, char *base) {

     int x = 0;
     char filename[160];

     /* initialize every starting point to NULL, so that we can tell
	if there's ever been something added to it. */
     
     start_email = NULL;
     start_ftp = NULL;
     start_phone = NULL;
     start_telnet = NULL;
     start_isdn = NULL;
     start_other = NULL;

     /* go through every file, and parse it while blindly cramming it
	into the right linked list. */

     while(filenames[x] != NULL) {
	  mangle_listfile(filenames[x]);
	  printf("Completed processing of %s\n", filenames[x]);
	  x++;
     }

#ifdef NL2K_DEBUG
     printf("%d nodelists processed, entering sort/write phase.\n",
	    x);
#endif

     /* sort everything into a coherent order before we make the
	index. */     
     sort_all_links();
     
     /* now, build_index() creates all of the files on disk... */

     build_index(base);

     /* hopefully, we're done, and it's all OK now. */

     return;

}

/* go through one single file, shoving every entry into the desired
   linked list. */

void mangle_listfile(char *filename) {

     int listfd;
     FILE *listfp;
     tNLDat *old;

     char listline[160]; /* hopefully no line is > 160 chars! */
     
     if((listfd = open(filename, O_RDONLY)) == -1) {
          printf("Couldn't open %s, skipping...\n", filename);
          return;
     }
     
     /* grrrrrrr....  why does nothing agree on what an end of line
	is?  it's unfortunate, and it makes our task more difficult.
	I originally wrote this with mmap(), which made life easier,
	but I'm shooting for some portability here, so, mmap() is
	right out.  We try to brute force through the lines with
	fgets() and see if the end of line characters fuck things up
	for us or not.  Your system's agreement with this code may
	vary, depending on how it likes gratuitous \r's or \n's
	floating around. */

     /* the original mmap code is the reason for having both listfd
	and listfp.  I'm not stdio's biggest fan, but we're trusting
	it here because we feel like it. */

     if((listfp = fdopen(listfd, "r")) == NULL) {
	  printf("Couldn't complete open of %s, skipping...\n",
		 filename);
	  close(listfd);
	  return;
     }
     
/*     old = (tNLDat *)malloc(sizeof(tNLDat)); */

     while(fgets(listline, 160, listfp) != NULL) {

	  /* variables for wrappers that we might have to chuck in the
	     linked lists. personally, I think that the way "other" vs
	     "normal" formats is done here can bite my wire.  3 more
	     bytes per node is negiligable, with
	     1) the size of the rest of the structure - 176 bytes for
	     a "normal" node versus 179 for an "other".  3 bytes is
	     less than 2% of the "normal" structure.  a minimal
	     increase.
	     2) modern storage spaces - even with a 100k node network,
	     one only adds 300k to the storage space (which, assuming
	     a network of all "normal" nodes (best case for storage
	     space) is goes from a 17600000 byte file to a 17900000
	     byte file...  hmmm...  looks even less significant now.
	     
	     but, it makes life more difficult for
	     programmers... making the decision for which to use when
	     compiling isn't bad, we'd have to check the string for
	     the connection type anyway to decide which file to store
	     it to.  However, on a read, *2* totally seperate sets of
	     calls need to exist, with almost exactly duplicated code,
	     need to exist, in order to handle "other" versus
	     "normal".  (granted, creative conditionals can minimize
	     some things, but when you've gotta return the right
	     structure at the end of a function, there's gotta be 2
	     calls, however you slice it.)  it's still less
	     brain-damaged than a lot of FTSC specs, tho. 

	     trap escaped here by storing everything into a tNLDat
	     initially, and then memcpy(), memcpy(), memcpy() when 
	     it's an "other".  some speed/elegance gain here.
*/
	  
	  /* tirade mode off. */

	  tNLDat *cur;

	  /* variables for each thing we read.  We'd be a fool to
	     write them directly into a structure, we don't know which
	     fucking one we're gonna use!  let's watch the machine
	     allocate the world twice... */

	  char addr[60];
          char cflag[3];
          char SiteName[31];
          char SiteLoc[31];
          char SysName[31];
          char Connection[31];
          char CapFlags[31];

	  
	  /* ignore any line which contains a comment, indicates start
	     of file, or has the DOS EOF character on it. */
	  
	  if((listline[0] == ';') || (listline[0] == ':') ||
	     (listline[0] == 26))
	       continue;

	  /* alright, this means we're gonna get something.  allocate
	     ram to put it all in. */

	  cur = (tNLDat*) malloc(sizeof(tNLDat));

	  /* ok, real line lies below.  time for some fun. */
	  
	  if(sscanf(listline,
		    "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%s",
                    addr, SiteName, SiteLoc, SysName, cflag,
                    Connection, CapFlags) < 7) {
               printf("Incomplete nodelist line (not enough fields):\n %s\n",
		      listline);
               continue;
	  }

          /* a '.' for any field means it's the same as the last one.
             so, don't change it if there is. */

	  /* we take an easier way out -- the old values are stored in
	     a tNLDat, so we don't have to duplicate all this code. */

	  /* we also make the assumption that the connection type will
	     never be duplicated. */

          if(SiteName[0] == '.')
               strcpy(SiteName, old->Connect.SiteName);
          if(SiteLoc[0] == '.')
               strcpy(SiteLoc, old->Connect.SiteLoc);
          if(SysName[0] == '.')
               strcpy(SysName, old->Connect.SysName);
          if(Connection[0] == '.')
               strcpy(Connection, old->Connect.Connection);
          if(CapFlags[0] == '.')
               strcpy(CapFlags, old->Connect.CapFlags);


	  strcpy(cur->Connect.SiteName, SiteName);
	  strcpy(cur->Connect.SiteLoc, SiteLoc);
	  strcpy(cur->Connect.SysName, SysName);
	  strcpy(cur->Connect.Connection, Connection);
	  strncpy(cur->Connect.CapFlags, CapFlags, 30);
	  
	  
	  /* alright, now break up "addr" into a valid address
             entry. */

	  /* ah, same address we had before.  So, let's copy the
	     relevant stuff into the new structure. */

          if(addr[0] == '.') 
	       memcpy(&cur->Address, &old->Address, sizeof(tAddress));

	  /* ah, piss, it's something new. :) */
	  else {
	       strcpy(cur->Address.Domain,
		      old->Address.Domain);
	       
	       /* break it on down, and see where our fun can
		  begin. */

	       if(strchr(addr, ':') != NULL) { /* it's a full zoned
						  addr. */
		    /* we make the assumption that domain entries only
		       appear on fully zoned lines.  we might be
		       wrong. */
		    
		    char domain[30];
		    int flds;
		    
		    /* is there a point there? */

		    if(strchr(addr, '.') != NULL)
			 flds = sscanf(listline, "%hd:%hd/%hd.%hd@%[^,]",
				       &cur->Address.Zone,
				       &cur->Address.Net,
				       &cur->Address.Node,
				       &cur->Address.Point, domain);
		    else {
			 flds = sscanf(listline, "%hd:%hd/%hd%[^,]",
				       &cur->Address.Zone,
				       &cur->Address.Net,
				       &cur->Address.Node, domain);
			 flds++; /* act like there was a point. */
			 cur->Address.Point = 0;
		    }
		    
		    /* now, if that line contained a domain, let's do
		       something with it, or fetch the old. */
		    
		    if(flds == 5)
			 strcpy(cur->Address.Domain, domain);

	       } else if(strchr(addr, '/') != NULL) { /* contains net,
							 no
							 zone */
		    cur->Address.Zone = old->Address.Zone;
		    
		    /* is there a point there? */
		    if(strchr(addr, '.') != NULL)
			 sscanf(listline, "%hd/%hd.%hd", &cur->Address.Net,
				&cur->Address.Node, &cur->Address.Point);
		    else {
			 sscanf(listline, "%hd/%hd", &cur->Address.Net,
				&cur->Address.Node);
			 cur->Address.Point = 0;
		    
		    }

	       } else { /* it's just a net. */
		    
		    cur->Address.Zone = old->Address.Zone;
		    cur->Address.Net = old->Address.Net;

		    /* is there a point there? */
		    if(strchr(addr, '.') != NULL)
			 sscanf(listline, "%hd.%hd", &cur->Address.Node,
				&cur->Address.Point);
		    else {
			 sscanf(listline, "%hd", &cur->Address.Node);
			 cur->Address.Point = 0;
		    }

	       } /* now, if it's just a point, I think we fucked up.
		    n2000.003 is silent on the point of what points,
		    and point alone look like.  We home they look like
		    net.point, in which case, we caught them. */

	  } /* and now, after much crap, we've mangled the address
	       into something sensible. */

#ifdef NL2K_DEBUG
	  printf("parsed line of %d:%d/%d.%d@%s\n", cur->Address.Zone,
		 cur->Address.Net, cur->Address.Node,
		 cur->Address.Point, cur->Address.Domain);
#endif

	  /* set the old to the right thing for us... */
	  old = cur;

	  /* alright, now the tNLDat should be complete. Now identify
	     the type, and then shove it on top on the list. */

	  if(!strcasecmp("CP", cflag)) { 
	       struct tNLDat_wrap *this;
	       this = (struct tNLDat_wrap *)malloc(sizeof(struct
							  tNLDat_wrap));
	       this->data = cur;
	       this->next = start_phone;
	       start_phone = this;
	  } else if(!strcasecmp("CF", cflag)) {
               struct tNLDat_wrap *this;
	       this = (struct tNLDat_wrap *)malloc(sizeof(struct
							  tNLDat_wrap));
               this->data = cur;
               this->next = start_ftp;
               start_ftp = this;
          } else if(!strcasecmp("CE", cflag)) {
               struct tNLDat_wrap *this;
	       this = (struct tNLDat_wrap *)malloc(sizeof(struct
							  tNLDat_wrap));
               this->data = cur;
               this->next = start_email;
               start_email = this;
	  } else if(!strcasecmp("CT", cflag)) {
               struct tNLDat_wrap *this;
	       this = (struct tNLDat_wrap *)malloc(sizeof(struct
							  tNLDat_wrap));
               this->data = cur;
               this->next = start_telnet;
               start_telnet = this;
	  } else if(!strcasecmp("CI", cflag)) {
               struct tNLDat_wrap *this;
	       this = (struct tNLDat_wrap *)malloc(sizeof(struct
							  tNLDat_wrap));
               this->data = cur;
               this->next = start_isdn;
               start_isdn = this;
	  } else {
	       /* ahhhhh.  the lovely "other" catagory */
               struct tNLDat_owrap *this;
	       this = (struct tNLDat_owrap *)malloc(sizeof(struct
							  tNLDat_owrap));
               this->data = (tNLODat *)malloc(sizeof(tNLODat));
	       memcpy(&this->data->Address, &cur->Address,
		      sizeof(tAddress));
	       strcpy(this->data->ConnFlag, cflag);
	       this->data->NodeFlags = cur->NodeFlags;
	       memcpy(&this->data->Connect, &cur->Connect,
		      sizeof(tConnect));
	       /* I think that takes care of getting everything to
		  where it needs to be... */
	       free(cur); /* don't need that anymore... */

	       this->next = start_other;
               start_other = this;
	  }

	  /* and that, baby, concludes one fucking line. */

     }

     fclose(listfp);
     close(listfd); /* is this necessary? */
	       
}

/* the function which hands every set of linked lists to the sort
   procedure.  Woooha. */

void sort_all_links(void) {

     /* here's a perfect example of when 2 code forks need to exist in
	order to handle "normal" and "other".  grrrrrrr.... */

     normal_sort(&start_email);
     normal_sort(&start_ftp);
     normal_sort(&start_phone);
     normal_sort(&start_telnet);
     normal_sort(&start_isdn);
     other_sort(&start_other);

     /* fun, baby */

}

/* here's the sort routine which handles the "normal" sorting.  it
   does an iterative swapping sort until no more need to be done.
   this might not be the fastest, but I bet it takes the least ram */

void normal_sort(struct tNLDat_wrap **begin) {

     struct tNLDat_wrap *prev, *one, *two;
     int changed = 1;

     while(changed == 1) {
	  changed = 0;
	  prev = NULL;
	  one = *begin;
     	  
	  /* run through this shit, swapping them every time it's out
	     of order.  ignore domain. */
	  
	  while((one != NULL) && ((two = one->next) != NULL)) {
	       
	       int comp;
	       	       
	       /* compare the 2 addresses, to see which one is less.
		  a 1 means that 2 is less, and swapping is
		  necessary. */
	       comp = addr_comp(&one->data->Address, &two->data->Address);
	       if(comp == 1) {
		    /* are we at the beginning? */
		    if(prev == NULL) 
			 *begin = two;
		    else
			 prev->next = two;
		    one->next = two->next;
		    two->next = one;
		    prev = two; /* two now comes before one */
		    changed = 1;
	       } else { /* nothing's changed, move along, there's
			   nothing to see here, please disperse.. */
		    prev = one;
		    one = two;

	       }

	       /* comparison complete.  keep moving. */
	  }
	  /* iteration complete, try again if need be. */

     }

     /* and nothing changed that time!  weeeeeee're outta here. */

}

/* here's the code that handles the sorting for an other.  note well
   that only 2 bytes have been inserted compared to above. */

void other_sort(struct tNLDat_owrap **begin) {

     struct tNLDat_owrap *prev, *one, *two;
     int changed = 1;

     while(changed == 1) {
	  changed = 0;
	  prev = NULL;
	  one = *begin;
     	  
	  /* run through this shit, swapping them every time it's out
	     of order.  ignore domain. */

	  while((one != NULL) && ((two = one->next) != NULL)) {
	       
	       int comp;
	       	       
	       /* compare the 2 addresses, to see which one is less.
		  a 1 means that 2 is less, and swapping is
		  necessary. */
	       comp = addr_comp(&one->data->Address, &two->data->Address);
	       if(comp == 1) {
		    /* are we at the beginning? */
		    if(prev == NULL) 
			 *begin = two;
		    else
			 prev->next = two;
		    one->next = two->next;
		    two->next = one;
		    prev = two; /* two now comes before one */
		    
		    changed = 1;
	       } else { /* nothing's changed, move along, there's
			   nothing to see here, please disperse.. */
		    prev = one;
		    one = two;
	       }

	       /* comparison complete.  keep moving. */
	  }
	  /* iteration complete, try again if need be. */

     }
     
     /* and nothing changed that time!  weeeeeee're outta here. */

}

/* address comparison.  -1 = first is lower than second, 0 = equal, 1
   = first is greater than second. */

/* disregards domains.  Also uses comparisons in a wildly inefficient
   way. */

int addr_comp(tAddress *first, tAddress *second) {

     if(first->Zone > second->Zone)
	  return 1;
     if(first->Zone < second->Zone)
	  return -1;

     if(first->Net > second->Net)
	  return 1;
     if(first->Net < second->Net)
	  return -1;

     if(first->Node > second->Node)
	  return 1;
     if(first->Node < second->Node)
	  return -1;

     if(first->Point > second->Point)
	  return 1;
     if(first->Point < second->Point)
	  return -1;

     /* if we haven't returned by now, everything's been equal. */

     return 0;

}

/* welp.  this one goes through every list, and creates the index
   entries for the stuff in it. I don't see as the index entries need
   to be in a particular order, really, though they will be, kinda,
   but not much. */

void build_index(char *base) {

     struct tNLIdx_wrap *start_index;
     struct tNLIdx_wrap *cur, *working;
     struct tNLDat_owrap *ocur;
     char filename[160];
     int fd;

     start_index = NULL;

     add_to_index(&start_index, start_email, NL_EMAIL);
     add_to_index(&start_index, start_ftp, NL_FTP);
     add_to_index(&start_index, start_phone, NL_PHONE);
     add_to_index(&start_index, start_telnet, NL_TELNET);
     add_to_index(&start_index, start_isdn, NL_ISDN);
     add_to_index_other(&start_index, start_other); /* aaaaaagh! */

     sprintf(filename, "%s/n2kidx.dat", base);
     if((fd = open(filename, O_TRUNC | O_WRONLY | O_CREAT, S_IRWXU |
		   S_IRWXG, S_IRWXO)) == -1) {
	  printf("Doh!  couldn't open %s, you just lost your compile run.\n",
		 filename);
	  return;
     }

     cur = start_index;
     while(cur != NULL) {
	  write(fd, cur->data, sizeof(tNLIdx));
	  cur = cur->next;
     }
     close(fd);

     sprintf(filename, "%s/n2kemail.dat", base);
     write_normal(start_email, filename);

     sprintf(filename, "%s/n2kftp.dat", base);
     write_normal(start_ftp, filename);

     sprintf(filename, "%s/n2kphone.dat", base);
     write_normal(start_phone, filename);

     sprintf(filename, "%s/n2ktelnet.dat", base);
     write_normal(start_telnet, filename);

     sprintf(filename, "%s/n2kisdn.dat", base);
     write_normal(start_isdn, filename);

     /* and now write out the "other" types.  virtually identical code
	appears below in start_normal. */
     
     sprintf(filename, "%s/n2kother.dat", base);
     if((fd = open(filename, O_TRUNC | O_WRONLY | O_CREAT, S_IRWXU |
                   S_IRWXG, S_IRWXO)) == -1) {
          printf("Doh!  couldn't open %s, you just lost a hunk of nodes.\n",
                 filename);
	  return;
     }
     
     ocur = start_other;
     while(ocur = NULL) {
	  write(fd, ocur->data, sizeof(tNLODat));
	  ocur = ocur->next;
     }
     close(fd);

     return;

}

void write_normal(struct tNLDat_wrap *begin, char *filename) {

     int fd;
     struct tNLDat_wrap *cur;

     if((fd = open(filename, O_TRUNC | O_WRONLY | O_CREAT, S_IRWXU |
                   S_IRWXG, S_IRWXO)) == -1) {
          printf("Doh!  couldn't open %s, you just lost a hunk of nodes.\n",
                 filename);
          return;
     }

     cur = begin;

     while(cur != NULL) {
          write(fd, cur->data, sizeof(tNLDat));
          cur = cur->next;
     }
     close(fd);

     return;


}

/* adds a linked list to the index stuff.  type is defined as above.
   other, naturally, doesn't count. */

void add_to_index(struct tNLIdx_wrap **begin, struct tNLDat_wrap
	     *begin_entries, int type) {

     struct tNLIdx_wrap *cur_index;
     struct tNLDat_wrap *cur_entry;
     int depth;

     /* no entries of this type, piss off. */

     if(begin_entries == NULL) 
	  return;

     cur_entry = begin_entries;
     
     cur_index = *begin;
     
     /* a depth++ will strike soon, we want 0 on the first pass. */
     depth = -1;
     
     while(cur_entry != NULL) {
	  struct tNLIdx_wrap *lookup;
	  depth++;

	  /* see if there's already an entry for this net/node */
	  lookup = find_index(*begin, &cur_entry->data->Address);
	  if(lookup == NULL) { /* create one */

	       struct tNLIdx_wrap *new_idx;
	       
	       new_idx = (struct tNLIdx_wrap *)malloc(sizeof(struct
							     tNLIdx_wrap));

	       new_idx->data = (tNLIdx *)malloc(sizeof(tNLIdx));
	       new_idx->data->Zone = cur_entry->data->Address.Zone;
	       new_idx->data->Net = cur_entry->data->Address.Net;
	       
	       /* n2000cf.003 is silent on what to do here, so I'm
		  initializing everything to 65535, and then replacing it
		  when an entry lives there. */
	       
	       new_idx->data->EmailIdx = 65535;
	       new_idx->data->FTPIdx = 65535;
	       new_idx->data->PhoneIdx = 65535;
	       new_idx->data->TelnetIdx = 65535;
	       new_idx->data->ISDNIdx = 65535;
	       new_idx->data->OtherIdx = 65535;
	       
	       
	       /* but which kind of record is this? */
	       switch(type) {
	       case NL_EMAIL:
		    new_idx->data->EmailIdx = depth;
		    break;
	       case NL_FTP:
		    new_idx->data->FTPIdx = depth;
		    break;
	       case NL_PHONE:
		    new_idx->data->PhoneIdx = depth;
		    break;
	       case NL_TELNET:
		    new_idx->data->TelnetIdx = depth;
		    break;
	       case NL_ISDN:
		    new_idx->data->ISDNIdx = depth;	
		    break;
	       default:
		    printf("Invalid type passed to index creator!\n");
		    printf("Compiler Fault - You shouldn't see this.\n");
		    break;
	       }
	       
	       /* now, merge our newly created entry into the
		  nodelist. */
	       
	       new_idx->next = *begin;
	       *begin = new_idx;
	       cur_entry = cur_entry->next;
	       continue;
	  }

	  /* zone/net does exist in index, and we've got it in lookup
	     which makes things rather trivial. */

	  /* see if there's already a definition for our type, and, if
	     not, add where wer are now.  if there is, we're not the
	     first zone/node entry here. */

	  switch(type) {
	  case NL_EMAIL:
	       if(lookup->data->EmailIdx == 65535)
                    lookup->data->EmailIdx = depth;
	       break;
	  case NL_FTP:
	       if(lookup->data->FTPIdx == 65535)
                    lookup->data->FTPIdx = depth;
	       break;
	  case NL_PHONE:
	       if(lookup->data->PhoneIdx == 65535)
                    lookup->data->PhoneIdx = depth;
	       break;
	  case NL_TELNET:
	       if(lookup->data->PhoneIdx == 65535)
                    lookup->data->TelnetIdx = depth;
	       break;
	  case NL_ISDN:
	       if(lookup->data->PhoneIdx == 65535)
                    lookup->data->ISDNIdx = depth;
	       break;
	  default:
	       printf("Invalid type passed to index creator!\n");
	       printf("Compiler Fault - You shouldn't see this.\n");
	       break;
	  }

	  /* now, either way, we've added the depth of this net/node
	     if it was necessary.  good. */
	  
	  cur_entry = cur_entry->next;
     }

     return;

}

/* the "other" version of add_to_index...  at least it's shorter as we
   know our type. */

void add_to_index_other(struct tNLIdx_wrap **begin, struct tNLDat_owrap
	     *begin_entries) {

     struct tNLIdx_wrap *cur_index;
     struct tNLDat_owrap *cur_entry;
     int depth;

     /* no entries of this type, piss off. */

     if(begin_entries == NULL) 
	  return;

     cur_entry = begin_entries;
     
     cur_index = *begin;
     
     /* a depth++ will strike soon, we want 0 on the first pass. */
     depth = -1;
     
     while(cur_entry != NULL) {
	  struct tNLIdx_wrap *lookup;
	  depth++;

	  /* see if there's already an entry for this net/node */
	  lookup = find_index(*begin, &cur_entry->data->Address);
	  if(lookup == NULL) { /* create one */
	       static struct tNLIdx_wrap *new_idx;
	       
	       new_idx = (struct tNLIdx_wrap *)malloc(sizeof(struct
							     tNLIdx_wrap));

	       new_idx->data = (tNLIdx *)malloc(sizeof(tNLIdx));
	       new_idx->data->Zone = cur_entry->data->Address.Zone;
	       new_idx->data->Net = cur_entry->data->Address.Net;

	       /* n2000cf.003 is silent on what to do here, so I'm
		  initializing everything to 65535, and then replacing it
		  when an entry lives there. */
	       
	       /* except for other, which becomes the depth. */

	       new_idx->data->EmailIdx = 65535;
	       new_idx->data->FTPIdx = 65535;
	       new_idx->data->PhoneIdx = 65535;
	       new_idx->data->TelnetIdx = 65535;
	       new_idx->data->ISDNIdx = 65535;
	       new_idx->data->OtherIdx = depth;
	       
	       /* now, merge our newly created entry into the
		  nodelist. */
	       
	       new_idx->next = *begin;
	       *begin = new_idx;
	       cur_entry = cur_entry->next;
	       continue;
	  }

	  /* zone/net does exist in index, and we've got it in lookup
	     which makes things rather trivial. */

	  /* see if there's already a definition for our type, and, if
	     not, add where wer are now.  if there is, we're not the
	     first zone/node entry here. */
	  
	       if(lookup->data->OtherIdx == 65535)
                    lookup->data->OtherIdx = depth;
	       break;

	  /* now, either way, we've added the depth of this net/node
	     if it was necessary.  good. */


     }

     return;

}

/* walks along the index and looks for a zone/net pair, and returns a
   pointer to it so we can modify it. */

struct tNLIdx_wrap *find_index(struct tNLIdx_wrap *begin, tAddress *lookup) {

     struct tNLIdx_wrap *cur;

     cur = begin;

     while(cur != NULL) {
	  if((lookup->Zone == cur->data->Zone) && (lookup->Net ==
						   cur->data->Net)) 
	       return cur;
	  cur = cur->next;
     }

     /* guess we never found it. */

     return NULL;

}
